<?php

namespace Drupal\send_request\Plugin;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Plugin for sending data to meDRA endpoint.
 *
 * @package Drupal\send_request\Plugin
 */
class MeDRAPlugin {
  use StringTranslationTrait;

  private $endpointUrl = '';
  private $username = '';
  private $password = '';
  private $developEndpointUrl = '';
  private $developUsername = '';
  private $developPassword = '';
  private $developMode = FALSE;

  /**
   * Configurations.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack object.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   */
  public function __construct(ConfigFactory $config_factory, LoggerChannelFactoryInterface $logger_factory, TranslationInterface $string_translation, RequestStack $request_stack, AccountInterface $current_user) {
    $this->config = $config_factory->getEditable('send_request.settings');
    $this->logger = $logger_factory->get('send_request');
    $this->stringTranslation = $string_translation;
    $this->currentUser = $current_user;
    $this->endpointUrl = $this->config->get('medra_url');
    $this->username = $this->config->get('medra_user');
    $this->password = $this->config->get('medra_password');
    $this->developEndpointUrl = $this->config->get('medra_test_url');
    $this->developUsername = $this->config->get('medra_test_user');
    $this->developPassword = $this->config->get('medra_test_password');
    $currentHost = $request_stack->getCurrentRequest()->getHttpHost();
    if (strpos($currentHost, '.docker') !== FALSE || strpos($currentHost, 'localhost') !== FALSE || strpos($currentHost, 'ddev') !== FALSE) {
      $this->logger->notice('Plugin <strong>MeDRAPlugin</strong> ist im Testmodus.');
      $this->developMode = TRUE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('config.factory'),
      $container->get('logger.factory'),
      $container->get('string_translation'),
      $container->get('request_stack'),
      $container->get('current_user')
    );
  }

  /**
   * Get endpoint.
   */
  public function getEndpoint() {
    if ($this->developMode) {
      if ($this->developEndpointUrl != '') {
        $this->logger->notice('Plugin <strong>MeDRAPlugin</strong> use the test URL <strong>' . $this->developEndpointUrl . '</strong>');
        return $this->developEndpointUrl;
      }
    }
    return $this->endpointUrl;
  }

  /**
   * Get username.
   */
  public function getUsername() {
    if ($this->developMode) {
      return $this->developUsername;
    }
    return $this->username;
  }

  /**
   * Get password.
   */
  public function getPassword() {
    if ($this->developMode) {
      return $this->developPassword;
    }
    return $this->password;
  }

  /**
   * Get method.
   */
  public function getMethod() {
    return 'POST';
  }

  /**
   * Get options.
   */
  public function getOptions() {
    return [
      'http_errors' => FALSE,
      'headers' => [
        'Content-Language' => strtolower($this->config->get('country_code')),
      ],
      'auth' => [
        $this->getUsername(),
        $this->getPassword(),
      ],
    ];
  }

  /**
   * Create data.
   */
  public function createData($node, $update) {
    // Check if we have bibsite.
    if (!$node->hasField('field_bibcite')) {
      return;
    }
    $bibciteEntities = $node->field_bibcite->referencedEntities();
    if (!isset($bibciteEntities[0])) {
      return;
    }

    // Bibcite data.
    $bibciteEntity = $bibciteEntities[0];
    // Authors.
    foreach ($bibciteEntity->author->referencedEntities() as $authorRef) {
      $authors[] = $authorRef->get('first_name')->value . ' ' . $authorRef->get('last_name')->value;
    }
    // Current user data.
    $currentUserAccount = $this->currentUser->getAccount();
    // Absolute url.
    $nodeUrl = $node->toUrl();
    $nodeUrl->setAbsolute();
    // JournalIssueDesignation.
    $journalIssueDesignation = trim($bibciteEntity->bibcite_secondary_title->value) . ' ' . trim($bibciteEntity->bibcite_volume->value);
    $firstPageNumber = 0;
    $lastPageNumber = 0;
    if (trim($bibciteEntity->bibcite_pages->value) != '') {
      $journalIssueDesignation .= ', ' . trim($bibciteEntity->bibcite_pages->value);
      // Get pages.
      if (strpos($bibciteEntity->bibcite_pages->value, '-') !== FALSE) {
        $pagesExplode = explode('-', trim($bibciteEntity->bibcite_pages->value));
        $firstPageNumber = (int) trim($pagesExplode[0]);
        $lastPageNumber = (int) trim($pagesExplode[1]);
      }
    }
    // Publication date.
    $dateArray = explode('/', trim($bibciteEntity->bibcite_date->value));
    $publicationDate = new \DateTime();
    $publicationDate->setDate($dateArray[1], $dateArray[0], 1);
    // ProductForm, if news4vets (79) than JB.
    $productForm = 'JD';
    $typesOfArticle = $node->field_art_des_artikels->getValue();
    foreach ($typesOfArticle as $typeOfArticle) {
      if ($typeOfArticle['target_id'] == 79) {
        $productForm = 'JB';
        break;
      }
    }
    // Get Language.
    $publishLang = 'eng';
    $bibciteLangValue = trim($bibciteEntity->bibcite_lang->value);
    if ($bibciteLangValue !== 'English' && $bibciteLangValue !== '') {
      // The first three lowercased letters.
      $publishLang = strtolower(substr($bibciteLangValue, 0, 3));
    }
    // Create data array.
    $content = [
      'Header' => [
        /*'NotificationResponse' => '02',*/
        'FromCompany' => $this->config->get('company'),
        'FromEmail' => $currentUserAccount->getEmail(),
        'ToCompany' => 'mEDRA',
        'SentDate' => date('Ymd'),
      ],
      'DOISerialArticleVersion' => [
        'NotificationType' => ($update === FALSE || $this->developMode === TRUE) ? '07' : '06',
        'DOI' => trim($bibciteEntity->bibcite_doi->value),
        'DOIWebsiteLink' => $nodeUrl->toString(),
        'RegistrantName' => $this->config->get('medra_registrant_name'),
        'SerialPublication' => [
          'SerialWork' => [
            'Title' => [
              'TitleType' => '01',
              'TitleText' => trim($bibciteEntity->bibcite_secondary_title->value),
            ],
            'Publisher' => [
              'PublishingRole' => '01',
              'PublisherName' => trim($bibciteEntity->bibcite_publisher->value),
            ],
            'CountryOfPublication' => $this->config->get('country_code'),
          ],
          'SerialVersion' => [
            'ProductIdentifier' => [
              'ProductIDType' => '07',
              'IDValue' => trim($bibciteEntity->bibcite_issn->value),
            ],
            'ProductForm' => $productForm,
          ],
        ],
        'JournalIssue' => [
          'JournalIssueDesignation' => $journalIssueDesignation,
          'JournalIssueDate' => [
            'DateFormat' => '00',
            'Date' => $publicationDate->format('Ymd'),
          ],
        ],
        'ContentItem' => [
          'TextItem' => [
            'PageRun' => [
              'FirstPageNumber' => $firstPageNumber,
              'LastPageNumber' => $lastPageNumber,
            ],
          ],
          'Title' => [
            'TitleType' => '01',
            'TitleText' => trim($node->title->value),
          ],
          'Contributor' => [
            'ContributorRole' => 'A01',
            'PersonName' => implode(', ', $authors),
          ],
          'Language' => [
            'LanguageRole' => '01',
            'LanguageCode' => $publishLang,
          ],
          'PublicationDate' => $publicationDate->format('Ymd'),
        ],
      ],
    ];

    // Creating object of SimpleXMLElement.
    $xmlStartElement = '<?xml version="1.0" encoding="utf-8"?><ONIXDOISerialArticleVersionRegistrationMessage	xmlns="http://www.editeur.org/onix/DOIMetadata/1.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.editeur.org/onix/DOIMetadata/1.0 http://www.medra.org/schema/onix/DOIMetadata/1.0/ONIX_DOIMetadata_1.0.xsd"></ONIXDOISerialArticleVersionRegistrationMessage>';
    $xmlData = new \SimpleXMLElement($xmlStartElement);
    $this->arrayToXml($content, $xmlData);
    $xmlAsString = $xmlData->asXML();

    // Saving generated xml file.
    $fileName = 'upload' . $node->get('nid')->getValue()[0]['value'];

    $data = [
      'file' => [
        'name' => $fileName,
        'content' => $xmlAsString,
        'extension' => 'xml',
      ],
      'LANG' => strtolower($this->config->get('country_code')),
    ];

    return $data;
  }

  /**
   * Handle the response.
   *
   * @todo react on the response from meDRA, see:
   * @link https://www-medra-dev.medra.org/stdoc/mEDRA_WebService_HTTP_callback_User_Guidev1.0.pdf
   */
  public function handleResponse($response) {
    return $response;
  }

  /**
   * Bild the form.
   */
  public function buildForm($form, $config) {
    $form['medra'] = [
      '#type' => 'details',
      '#title' => 'meDRA data transmission',
      '#open' => TRUE,
    ];
    $form['medra']['medra_activated'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Data transmission activated'),
      '#description' => $this->t('After creating a node, an XML file is sent to https://www.medra.org/servlet/ws/upload.'),
      '#default_value' => $config->get('medra_activated'),
    ];
    $form['medra']['medra_registrant_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Registrant name'),
      '#description' => $this->t('Please enter the registrant name for the meDRA service.'),
      '#default_value' => $config->get('medra_registrant_name'),
    ];

    $form['medra']['medra_live'] = [
      '#type' => 'details',
      '#title' => $this->t('Live endpoint'),
      '#open' => TRUE,
    ];
    $form['medra']['medra_live']['medra_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#description' => $this->t('Please enter here the URL to which the content should be sent in live mode.'),
      '#default_value' => $config->get('medra_url'),
    ];
    $form['medra']['medra_live']['medra_user'] = [
      '#type' => 'textfield',
      '#title' => $this->t('User'),
      '#description' => $this->t('Please enter the user for the Live-URL here.'),
      '#default_value' => $config->get('medra_user'),
    ];
    $form['medra']['medra_live']['medra_password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#description' => $this->t('Please enter the password for the Live-URL here.'),
      '#default_value' => $config->get('medra_password'),
    ];

    $form['medra']['medra_test'] = [
      '#type' => 'details',
      '#title' => $this->t('Test endpoint'),
      '#open' => TRUE,
    ];
    $form['medra']['medra_test']['medra_test_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#description' => $this->t('Please enter a URL here that should only be used for testing. ATTENTION: this will not be used in live mode, even if it is set.'),
      '#default_value' => $config->get('medra_test_url'),
    ];
    $form['medra']['medra_test']['medra_test_user'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#description' => $this->t('Please enter the username for the test URL here.'),
      '#default_value' => $config->get('medra_test_user'),
    ];
    $form['medra']['medra_test']['medra_test_password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
      '#description' => $this->t('Please enter the password for the test URL here.'),
      '#default_value' => $config->get('medra_test_password'),
    ];

    $form['medra']['medra_content_types'] = [
      '#type' => 'textfield',
      '#title' => $this->t('For which content type(s)'),
      '#description' => $this->t('The data of which content types should be sent? Please separate with |.'),
      '#default_value' => $config->get('medra_content_types'),
    ];

    return $form;
  }

  /**
   * Validate the form.
   */
  public function validateForm(&$form, &$form_state) {
    // Build form for meDRA.
    if ($form_state->getValue('medra_activated') == 1) {
      // Check registrant name.
      if (trim($form_state->getValue('medra_registrant_name')) == '') {
        $form_state->setErrorByName('medra_registrant_name',
          $this->t('Registrant name must be set.')
        );
      }
      // Check live url.
      if (trim($form_state->getValue('medra_url')) == '') {
        $form_state->setErrorByName('medra_url',
          $this->t('URL under live endpoint must be set.')
        );
      }
      // Check live user.
      if (trim($form_state->getValue('medra_user')) == '') {
        $form_state->setErrorByName('medra_user',
          $this->t('User under live endpoint must be set.')
        );
      }
      // Check live password.
      if (trim($form_state->getValue('medra_password')) == '') {
        $form_state->setErrorByName('medra_password',
          $this->t('Password under live endpoint must be set.')
        );
      }
      // Check content type.
      if (trim($form_state->getValue('medra_content_types')) == '') {
        $form_state->setErrorByName('medra_content_types',
          $this->t('"For which content type(s)" must be set.')
        );
      }
    }
  }

  /**
   * Submit the form.
   */
  public function submitForm($config, $form_state) {
    $config
      ->set('medra_activated', $form_state->getValue('medra_activated'))
      ->set('medra_registrant_name', trim($form_state->getValue('medra_registrant_name')))
      ->set('medra_url', trim($form_state->getValue('medra_url')))
      ->set('medra_user', trim($form_state->getValue('medra_user')))
      ->set('medra_password', trim($form_state->getValue('medra_password')))
      ->set('medra_test_url', trim($form_state->getValue('medra_test_url')))
      ->set('medra_test_user', trim($form_state->getValue('medra_test_user')))
      ->set('medra_test_password', trim($form_state->getValue('medra_test_password')))
      ->set('medra_content_types', trim($form_state->getValue('medra_content_types')));

    return $config;
  }

  /**
   * Transform an array to XML.
   */
  private function arrayToXml($data, &$xmlData) {
    foreach ($data as $key => $value) {
      if (is_numeric($key)) {
        // Dealing with <0/>..<n/> issues.
        $key = 'item' . $key;
      }
      if (is_array($value)) {
        $subnode = $xmlData->addChild($key);
        $this->arrayToXml($value, $subnode);
      }
      else {
        $xmlData->addChild("$key", htmlspecialchars("$value"));
      }
    }
  }

}
