<?php

namespace Drupal\send_request;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Client sending the requests to the endpoints.
 */
class SendRequestClient {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * Construct a Schema client object.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   */
  public function __construct(ClientInterface $http_client, LoggerChannelFactoryInterface $logger_factory) {
    $this->httpClient = $http_client;
    $this->logger = $logger_factory->get('send_request');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client'),
      $container->get('logger.factory')
    );
  }

  /**
   * Send data to endpoint.
   *
   * @param mixed $endpointPlugin
   *   The plugin used to send the data to the endpoint.
   * @param array $data
   *   Data to send.
   *
   * @return mixed
   *   Returns response or false, if the response there is no valid response.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function sendDataToEndpoint($endpointPlugin, array $data) {
    // Get endpoint.
    $endPoint = $endpointPlugin->getEndpoint();
    // Get options, also username and password if is set.
    $options = $endpointPlugin->getOptions();
    // Get method.
    $method = $endpointPlugin->getMethod();

    // Set data to "form".
    if (count($data) > 0) {
      $options['multipart'] = [];
      $dataForLogging = '';
      foreach ($data as $name => $content) {
        if ($name == 'file') {
          $options['multipart'][] = [
            'name' => $data['file']['name'],
            'contents' => $data['file']['content'],
            'filename' => $data['file']['name'] . '.' . $data['file']['extension'],
          ];
          $dataForLogging .= $name . ' ' . $data['file']['name'] . ': ' . $data['file']['content'];
        }
        else {
          $options['multipart'][] = [
            'name' => $name,
            'contents' => $content,
          ];
          $dataForLogging .= $name . ': ' . $content;
        }
        $dataForLogging .= '<br />';
      }

      try {
        $response = $this->httpClient->request($method, $endPoint, $options);
        $code = $response->getStatusCode();
        if ($code == 200) {
          // Logging.
          $this->logger->notice('Plugin <strong>' . get_class($endpointPlugin) . '</strong> has sent the following data to <strong>' . $endPoint . '</strong>: ' . $dataForLogging);
        }
        else {
          // Logging.
          $this->logger->error('Plugin <strong>' . get_class($endpointPlugin) . '</strong> has sent the following data to <strong>' . $endPoint . '</strong>.<br />Status code: ' . $code . '<br />Data: ' . $dataForLogging . '<br />Body: ' . $response->getBody());
        }
        return $response;
      }
      catch (RequestException $e) {
        watchdog_exception('send_request', $e);
        return FALSE;
      }
    }
  }

}
